package com.example.volunteersapp;

import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.os.Bundle;
import android.text.format.DateFormat;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.Menu;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.TimePicker;
import android.widget.Toast;

import com.example.volunteersapp.ui.home.HomeFragment;
import com.google.android.material.navigation.NavigationView;

import androidx.annotation.NonNull;
import androidx.fragment.app.DialogFragment;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.AppCompatActivity;

import com.example.volunteersapp.databinding.ActivityDrawerBinding;

import java.util.Calendar;

public class Drawer extends AppCompatActivity
{

    private AppBarConfiguration mAppBarConfiguration;
    private ActivityDrawerBinding binding;

    private DatePickerDialog datePickerDialog;
    private static Button dateButton, timeFrom, timeTo;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        binding = ActivityDrawerBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        dateButton = findViewById(R.id.date);
        dateButton.setText(getTodayDate());

        timeFrom = findViewById(R.id.timeFrom);
        timeFrom.setText(getNow(0));

        timeTo = findViewById(R.id.timeTo);
        timeTo.setText(getNow(1));

        setSupportActionBar(binding.appBarDrawer.toolbar);
        binding.appBarDrawer.fab.setOnClickListener(new View.OnClickListener()
            {
                @Override
                public void onClick(View view)
                {
                    Toast.makeText(Drawer.this, "TADA", Toast.LENGTH_SHORT).show();
                }
            }
        );
        binding.appBarDrawer.fab.hide();

        DrawerLayout drawer = binding.drawerLayout;
        NavigationView navigationView = binding.navView;
        mAppBarConfiguration = new AppBarConfiguration.Builder(
                R.id.nav_home, R.id.nav_gallery, R.id.nav_slideshow)
                .setOpenableLayout(drawer)
                .build();
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_drawer);
        NavigationUI.setupActionBarWithNavController(this, navController, mAppBarConfiguration);
        NavigationUI.setupWithNavController(navigationView, navController);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        getMenuInflater().inflate(R.menu.drawer, menu);
        return true;
    }

    @Override
    public boolean onSupportNavigateUp()
    {
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment_content_drawer);
        return NavigationUI.navigateUp(navController, mAppBarConfiguration)
                || super.onSupportNavigateUp();
    }

    public static String getTodayDate()
    {
        Calendar cal = Calendar.getInstance();
        int year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH);
        month = month + 1;
        int day = cal.get(Calendar.DAY_OF_MONTH);

        return makeDateString(day, month, year);
    }

    public static String getNow(int plusHour)
    {
        Calendar cal = Calendar.getInstance();
        int hour = cal.get(Calendar.HOUR_OF_DAY);
        int minute = cal.get(Calendar.MINUTE);
        hour += plusHour;

        return makeTimeFormat(hour, minute);
    }

    public static String makeDateString(int day, int month, int year)
    {
        return day  + "." + month + "." + year;
    }

    public static String makeTimeFormat(int hour, int minute)
    {
        if (minute < 10)
        {
            return hour  + ":0" + minute ;
        }
        return hour  + ":" + minute ;
    }

    public void showTimePickerDialog(View v)
    {
        DialogFragment newFragment = new TimePickerFragment();
        TimePickerFragment.setID(getResources().getResourceEntryName( v.getId()));
        newFragment.show(getSupportFragmentManager(), "timePicker");
    }

    public void showDatePickerDialog(View v) {
        DialogFragment newFragment = new DatePickerFragment();
        DatePickerFragment.setID(getResources().getResourceEntryName( v.getId()));
        newFragment.show(getSupportFragmentManager(), "datePicker");
    }

    public static class TimePickerFragment extends DialogFragment implements TimePickerDialog.OnTimeSetListener
    {
        private static String ID;

        public static void setID(String layoutID)
        {
            ID = layoutID;
        }

        @Override
        public Dialog onCreateDialog(Bundle savedInstanceState)
        {
            final Calendar c = Calendar.getInstance();
            int hour = c.get(Calendar.HOUR_OF_DAY);
            int minute = c.get(Calendar.MINUTE);

            return new TimePickerDialog(getActivity(), this, hour, minute, DateFormat.is24HourFormat(getActivity()));
        }

        @Override
        public void onTimeSet(TimePicker timePicker, int hour, int minute)
        {
            int id = getResources().getIdentifier(ID, "id", getActivity().getPackageName());
            Button button = getActivity().findViewById(id);
            button.setText(makeTimeFormat(hour, minute));
        }
    }

    public static class DatePickerFragment extends DialogFragment implements DatePickerDialog.OnDateSetListener
    {
        private static String ID;

        public static void setID(String layoutID)
        {
            ID = layoutID;
        }
        @Override
        public Dialog onCreateDialog(Bundle savedInstanceState)
        {
            final Calendar c = Calendar.getInstance();
            int year = c.get(Calendar.YEAR);
            int month = c.get(Calendar.MONTH);
            int day = c.get(Calendar.DAY_OF_MONTH);

            return new DatePickerDialog(getActivity(), this, year, month, day);
        }

        public void onDateSet(DatePicker view, int year, int month, int day)
        {
            Log.d("TEST", "" + year);
            Log.d("TEST", "" + month);
            Log.d("TEST", "" + day);
            month = month + 1;
            String date = makeDateString(day, month, year);

            int id = getResources().getIdentifier(ID, "id", getActivity().getPackageName());
            Button dateButton = getActivity().findViewById(id);
            dateButton.setText(date);
        }
    }
}