package com.example.volunteersapp;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.biometric.BiometricPrompt;
import androidx.core.content.ContextCompat;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;
import com.example.volunteersapp.ui.model.User;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;


public class MainActivity extends AppCompatActivity
{

    EditText etName, etPassword;
    Button btnLogin;
    ProgressBar progressBar;
    ImageView fingerprint;

    private Executor executor;
    private BiometricPrompt biometricPrompt;
    private BiometricPrompt.PromptInfo promptInfo;

    private SharedPreferences sp;

    static public User userData = new User();

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        etName = findViewById(R.id.et_name);
        etPassword = findViewById(R.id.et_password);
        btnLogin = findViewById(R.id.submit);
        progressBar = findViewById(R.id.progressBar);
        fingerprint = findViewById(R.id.fingerprint);

        fingerprint.setVisibility(View.GONE);

        SharedPreferences spData = getSharedPreferences("Login", MODE_PRIVATE);

        String loginUserName = spData.getString("userName", null);

        if (loginUserName != null)
        {
            fingerprint.setVisibility(View.VISIBLE);
            sp = getSharedPreferences("Login", MODE_PRIVATE);

            userData.setUserID( spData.getInt("userID", 0));
            userData.setUserName(loginUserName);


//            SharedPreferences.Editor Ed = sp.edit();
//            Ed.putInt("userID", userData.getUserID() );
//            Ed.putString("userName", null);
//            Ed.apply();
        }

        executor = ContextCompat.getMainExecutor(this);
        biometricPrompt = new BiometricPrompt(MainActivity.this, executor, new BiometricPrompt.AuthenticationCallback()
            {
                @Override
                public void onAuthenticationError(int errorCode, @NonNull CharSequence errString)
                {
                    super.onAuthenticationError(errorCode, errString);
                }

                @Override
                public void onAuthenticationSucceeded(@NonNull BiometricPrompt.AuthenticationResult result)
                {
                    super.onAuthenticationSucceeded(result);
                    Toast.makeText(MainActivity.this, "Byl/a jste uspěšně přihlášen/a", Toast.LENGTH_SHORT).show();
                    startActivity(new Intent(getApplicationContext(), Drawer.class));
                }

                @Override
                public void onAuthenticationFailed() {
                    super.onAuthenticationFailed();
                    Toast.makeText(MainActivity.this, "Omlouváme se, něco se nepovedlo", Toast.LENGTH_SHORT).show();
                }
            }
        );

        promptInfo = new BiometricPrompt.PromptInfo.Builder()
            .setTitle("Přihlášení")
            .setSubtitle("(Přihlášení posledního přihlášeného účtu)")
            .setNegativeButtonText("Zavřít")
            .build();

        fingerprint.setOnClickListener(new View.OnClickListener()
            {
                @Override
                public void onClick(View view)
                {
                    biometricPrompt.authenticate(promptInfo);
                }
            }
        );

        btnLogin.setOnClickListener(new View.OnClickListener()
            {
                @Override
                public void onClick(View view)
                {
                    String name = etName.getText().toString().trim();
                    String password = etPassword.getText().toString().trim();

                    if (TextUtils.isEmpty(name))
                    {
                        etName.setError("Nutné zadat email");
                        return;
                    }


                    if (TextUtils.isEmpty(password))
                    {
                        etPassword.setError("Nutné zadat heslo");
                        return;
                    }

                    progressBar.setVisibility(View.VISIBLE);

                    String url = "http://volunteers-cz.mzf.cz/api/login";
                    StringRequest stringRequest =
                        new StringRequest(Request.Method.POST,
                            url,
                            new Response.Listener<String>()
                            {
                                @Override
                                public void onResponse(String response)
                                {
                                    try {
                                        JSONObject  responseJSON = new JSONObject(response);
                                        Toast.makeText(MainActivity.this, responseJSON.getString("msg"), Toast.LENGTH_SHORT).show();

                                        if (responseJSON.getBoolean("state"))
                                        {
                                            int userID = responseJSON.getInt("userID");

                                            userData.setUserID(userID);
                                            userData.setUserName(name);
                                            sp = getSharedPreferences("Login", MODE_PRIVATE);
                                            SharedPreferences.Editor Ed = sp.edit();

                                            Ed.putInt("userID", userData.getUserID() );
                                            Ed.putString("userName", name);
                                            Ed.apply();

                                            startActivity(new Intent(getApplicationContext(), Drawer.class));
                                        }
                                    } catch (JSONException e) {
                                        e.printStackTrace();
                                    }
                                }
                            },
                            new Response.ErrorListener() {
                                @Override
                                public void onErrorResponse(VolleyError error)
                                {
                                    Toast.makeText(MainActivity.this, error.toString(), Toast.LENGTH_SHORT).show();
                                }
                            }
                        )
                        {
                            @Override
                            protected Map<String, String> getParams()
                            {
                                Map<String, String> params = new HashMap<String, String>();
                                params.put("name", name);
                                params.put("password", password);

                                return params;
                            }
                        };
                    RequestQueue requestQue = Volley.newRequestQueue(MainActivity.this);
                    requestQue.add(stringRequest);


                    progressBar.setVisibility(View.GONE);
                }
            }
        );
    }
}