package com.example.volunteersapp.ui.gallery;

import static com.example.volunteersapp.MainActivity.userData;

import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;
import com.example.volunteersapp.R;
import com.example.volunteersapp.databinding.FragmentGalleryBinding;
import com.example.volunteersapp.ui.model.User;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Text;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class GalleryFragment extends Fragment {

    private FragmentGalleryBinding binding;

    private ListView listView;

    String programs[], from[], to[], descriptions[], date[];

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        GalleryViewModel galleryViewModel =
                new ViewModelProvider(this).get(GalleryViewModel.class);

        binding = FragmentGalleryBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        listView = (ListView) binding.volunteerList;


        String url = "http://volunteers-cz.mzf.cz/api/getAttendance";
        StringRequest stringRequest = new StringRequest(Request.Method.POST,
                        url,
                        new Response.Listener<String>()
                        {
                            @Override
                            public void onResponse(String response)
                            {
                                try {
                                    JSONObject responseJSON = new JSONObject(response);

                                    int length = responseJSON.getJSONArray("data").length();

                                    programs = new String[length];
                                    descriptions = new String[length];
                                    from = new String[length];
                                    to = new String[length];
                                    date = new String[length];

                                    JSONArray jsonArray;
                                    JSONObject jsonAttendance;
                                    for (int i = 0; i < length; i++)
                                    {
                                        jsonArray = responseJSON.getJSONArray("data");
                                        jsonAttendance = new JSONObject(jsonArray.getString(i));
                                        programs[i] = jsonAttendance.getString("program");
                                        descriptions[i] = jsonAttendance.getString("description");
                                        from[i] = jsonAttendance.getString("workFrom");
                                        to[i] = jsonAttendance.getString("workTo");
                                        date[i] = jsonAttendance.getString("date");
                                    }

                                    MyAdapter adapter = new MyAdapter(root.getContext(), programs, descriptions, from, to, date);

                                    listView.setAdapter(adapter);
                                } catch (JSONException e) {
                                    e.printStackTrace();
                                }
                            }
                        },
                        new Response.ErrorListener() {
                            @Override
                            public void onErrorResponse(VolleyError error)
                            {
                                Toast.makeText(root.getContext(), "Omlouváme se něco se nepovedlo, zkontrolujte data", Toast.LENGTH_SHORT).show();
                            }
                        }
                )
                {
                    @Override
                    protected Map<String, String> getParams()
                    {
                        Map<String, String> params = new HashMap<String, String>();

                        params.put("userID",  ""+userData.getUserID());

                        return params;
                    }
                };
        RequestQueue requestQue = Volley.newRequestQueue(root.getContext());
        requestQue.add(stringRequest);

        return root;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    static class MyAdapter extends ArrayAdapter<String>
    {
        Context context;
        String rProgram[];
        String rDescription[];
        String rFrom[];
        String rTo[];
        String rDate[];

        MyAdapter(Context c, String[] program, String[] description, String[] from, String[] to, String[] date)
        {
            super(c, R.layout.row, R.id.programValue, program);
            this.context = c;
            this.rProgram = program;
            this.rDescription = description;
            this.rFrom = from;
            this.rTo = to;
            this.rDate = date;

        }

        @NonNull
        @Override
        public View getView(int position, @Nullable View convertView, @NonNull ViewGroup parent) {
            LayoutInflater layoutInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

            View row = layoutInflater.inflate(R.layout.row, parent,false);
            TextView header = row.findViewById(R.id.programValue);
            TextView description = row.findViewById(R.id.description);
            TextView from = row.findViewById(R.id.workFromValue);

            header.setText(rProgram[position]);
            description.setText(rDescription[position]);
            from.setText(rFrom[position] + " - " + rTo[position] + " " + rDate[position]);

            return row;
        }
    }
}