package com.example.volunteersapp.ui.home;

import static com.example.volunteersapp.MainActivity.userData;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.android.volley.toolbox.Volley;
import com.example.volunteersapp.Drawer;
import com.example.volunteersapp.MainActivity;
import com.example.volunteersapp.R;
import com.example.volunteersapp.databinding.FragmentHomeBinding;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

public class HomeFragment extends Fragment
{

    private FragmentHomeBinding binding;

    FloatingActionButton add;
    EditText editTextUserName;
    Spinner dropdown;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState)
    {
        HomeViewModel homeViewModel =
                new ViewModelProvider(this).get(HomeViewModel.class);

        binding = FragmentHomeBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        editTextUserName = binding.userName;
        homeViewModel.getUserName().observe(getViewLifecycleOwner(), editTextUserName::setText);

        dropdown = binding.program;
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(root.getContext(),
                R.array.program_array, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        dropdown.setAdapter(adapter);

        add = binding.addActivity;

        add.setOnClickListener(new View.OnClickListener()
           {
               @Override
               public void onClick(View view)
               {
                   String userName = userData.getUserName();
                   int userID = userData.getUserID();

                   String programValue = binding.program.getSelectedItem().toString().trim();
                   String date = binding.date.getText().toString().trim();
                   String timeFrom = binding.timeFrom.getText().toString().trim();
                   String timeTo = binding.timeTo.getText().toString().trim();
                   String activity = binding.volunteerActivity.getText().toString().trim();

                   if (TextUtils.isEmpty(activity))
                   {
                       binding.volunteerActivity.setError("Nutné zadat email");
                       return;
                   }
                   String url = "http://volunteers-cz.mzf.cz/api/addAttendance";
                   StringRequest stringRequest =
                           new StringRequest(Request.Method.POST,
                                   url,
                                   new Response.Listener<String>()
                                   {
                                       @Override
                                       public void onResponse(String response)
                                       {
                                           try {
                                               JSONObject  responseJSON = new JSONObject(response);
                                               if (responseJSON.getBoolean("result"))
                                               {
                                                   Toast.makeText(root.getContext(), "Docházka byla úspěšně uložena", Toast.LENGTH_SHORT).show();
                                               } else {
                                                   Toast.makeText(root.getContext(), "Omlouváme se něco se nepovedlo, zkontrolujte data", Toast.LENGTH_SHORT).show();
                                               }
                                           } catch (JSONException e) {
                                               e.printStackTrace();
                                           }
                                       }
                                   },
                                   new Response.ErrorListener() {
                                       @Override
                                       public void onErrorResponse(VolleyError error)
                                       {
                                           Toast.makeText(root.getContext(), "Omlouváme se něco se nepovedlo, zkontrolujte data", Toast.LENGTH_SHORT).show();
                                       }
                                   }
                           )
                           {
                               @Override
                               protected Map<String, String> getParams()
                               {
                                   Map<String, String> params = new HashMap<String, String>();

                                   params.put("userName", userName);
                                   params.put("userID",  ""+userID);
                                   params.put("programValue", programValue);
                                   params.put("date", date);
                                   params.put("timeFrom", timeFrom);
                                   params.put("timeTo", timeTo);
                                   params.put("activity", activity);

                                   return params;
                               }
                           };
                   RequestQueue requestQue = Volley.newRequestQueue(root.getContext());
                   requestQue.add(stringRequest);

               }
           }
        );

        return root;
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }
}