<?php
namespace Classes\Attendance;

use Classes\Attendance\Manager as AManager;
use Classes\Attendance\Facade as AFacade;
use Classes\DateTimeUtil;
use Classes\Registration\Manager as RManager;
use Classes\Login\Manager as LManager;
use Classes\ExcelGenerator as EGenerator;
use Classes\StaticFunctions as Statics;
use PhpOffice\PhpSpreadsheet\Style\Alignment;

class reports
{

    private EGenerator $generator;
    private LManager $loginManager;
    private AFacade $attendanceFacade;
    private AManager $attendanceManager;

    public function __construct(AManager $attendanceManager, AFacade $attendanceFacade, LManager $loginManager,
                                EGenerator $generator)
    {
        $this->attendanceManager = $attendanceManager;
        $this->attendanceFacade = $attendanceFacade;
        $this->loginManager = $loginManager;
        $this->generator = $generator;
    }

    public function generateAttendanceReport(array $parameters) : void
    {
        $sheet = $this->generator->getSheet();
        $spreadsheet = $this->generator->getSpreadSheet();

        //Head

        $this->generator->setDefaultHeader($spreadsheet, $sheet, 'Vygenerovaná docházka');
        $this->generator->setDefaultTime($sheet);

        $sheet->getStyle('B1:I2')->applyFromArray($this->generator->getBorderStyleHeader());
        $column = 'B';
        $row = '3';
        $sheet->setCellValue($column.$row, 'Kdo');
        $sheet->setCellValue(++$column.$row, 'Měsíc');
        $sheet->setCellValue(++$column.$row, 'Program');
        $sheet->setCellValue(++$column.$row, 'Odpracované hodiny');
        $sheet->setCellValue(++$column.$row, 'Čas od');
        $sheet->setCellValue(++$column.$row, 'Čas do');
        $sheet->setCellValue(++$column.$row, 'Kdy');
        $sheet->setCellValue(++$column.$row, 'Náplň');
        $sheet->getStyle('B3:I3')->applyFromArray($this->generator->getBorderStyleLabels());

        //Body
        $column = 'B';
        $row = '4';

        $filters = [
            'program' => ($parameters['program'] != "null" && $parameters['program'] != "")? $parameters['program'] : null,
            'month' => $parameters['month'] != "null"? $parameters['month'] : null,
            'user' => $parameters['user'] != "null"? $parameters['user'] : null,
            'contract' => $parameters['contract'] != "null"? $parameters['contract'] : null,
            'year' => $parameters['year'] != "null"? $parameters['year'] : null
        ];

        $data = $this->attendanceManager->getAllForCoordinator(Statics::getOrganization(), $filters, true);

        $lastColumn = (3 + count($data));
        $sheet->getStyle('C4:CH'. $lastColumn)->getAlignment()->setHorizontal(Alignment::HORIZONTAL_CENTER);
        $sheet->getStyle('I4:I'. $lastColumn)->getAlignment()->setHorizontal(Alignment::HORIZONTAL_LEFT);
        $sheet->getStyle('B4:I'. $lastColumn)->applyFromArray($this->generator->getBorderStyleBody());
        $sum = 0;
        foreach ($data as $attendance)
        {
            $when = new \Classes\DateTimeUtil($attendance[AManager::DATE]);

            $sheet->setCellValue($column.$row, sprintf('%s %s (%s)',
                $attendance[RManager::FIRST_NAME], $attendance[RManager::SUR_NAME], $attendance[LManager::USER_NAME]));
            $sheet->setCellValue(++$column.$row, \Classes\DateTimeUtil::getMonth($attendance[AManager::MONTH]));
            $sheet->setCellValue(++$column.$row, AManager::PROGRAM_DICTIONARY[$attendance[AManager::PROGRAM]]);
            $sheet->setCellValue(++$column.$row, $attendance[AManager::WORK_HOURS]);
            $sheet->setCellValue(++$column.$row,
                \Classes\DateTimeUtil::getTimeFormat($attendance[AManager::WORK_FROM]));
            $sheet->setCellValue(++$column.$row,
                \Classes\DateTimeUtil::getTimeFormat($attendance[AManager::WORK_TO]));
            $sheet->setCellValue(++$column.$row, $when->getDateFormat());
            $sheet->setCellValue(++$column.$row, $attendance[AManager::DESCRIPTION]);
            $row++;
            $column = 'B';
            $sum = $sum + $attendance[AManager::WORK_HOURS];
        }

        //$sumRange = 'E4:E4'.$lastColumn;
        $sheet->setCellValue('D'.$row , 'Celkem:');
        $sheet->setCellValue('E'.$row , $sum);

        $now = new \Classes\DateTimeUtil();
        $this->generator->save($now->getDateFormat() . '_Dochazka');
    }

    public function generateWorkingReportStrict(array $parameters) : void
    {
        $parameterKey = key($parameters);
        $sheet = $this->generator->getSheet();
        $spreadsheet = $this->generator->getSpreadSheet();

        //Head
        $this->generator->setDefaultHeader($spreadsheet, $sheet, 'Vygenerované odpracované hodiny', 'D');
        $this->generator->setDefaultTime($sheet, 'D');

        $sheet->getStyle('B1:D2')->applyFromArray($this->generator->getBorderStyleHeader());
        $sheet->setCellValue('B3', 'Odpracované hodiny');

        $this->setParameters($parameters, $parameterKey);

        //Body
        $data = $this->attendanceFacade->getWorkingHourData(0, $parameters);

        $sheet->setCellValue('C3', $data['sum']);
        $sheet->setCellValue('D3', $data[$parameterKey]);

        $now = new \Classes\DateTimeUtil();
        $this->generator->save($now->getDateFormat() . '_OdpracovaneHodiny');
    }

    public function generateWorkingReportTable(array $parameters) :void
    {
        $parameterKey = key($parameters);
        $sheet = $this->generator->getSheet();
        $spreadsheet = $this->generator->getSpreadSheet();

        //Head
        $this->generator->setDefaultHeader($spreadsheet, $sheet, 'Vygenerovaná docházka', 'F');
        $this->generator->setDefaultTime($sheet, 'F');

        $sheet->getStyle('B1:F2')->applyFromArray($this->generator->getBorderStyleHeader());
        $column = 'B';
        $row = '3';

        switch ($parameterKey)
        {
            case 'year':
                $name = 'Rok';
            break;
            case 'userName':
                $name = 'Uživatel';
            break;
            case 'month':
                $name = 'Měsíc';
            break;
            default:
                $name = 'Program';
            break;
        }

        $sheet->setCellValue($column.$row, $name);
        $sheet->setCellValue(++$column.$row, 'Čas od');
        $sheet->setCellValue(++$column.$row, 'Čas do');
        $sheet->setCellValue(++$column.$row, 'Odpracované hodiny');
        $sheet->setCellValue(++$column.$row, 'Kdy');
        $sheet->getStyle('B3:F3')->applyFromArray($this->generator->getBorderStyleLabels());

        //Body
        $column = 'B';
        $row = '4';

        $this->setParameters($parameters, $parameterKey);
        $data = $this->attendanceFacade->getWorkingHourData(0, $parameters);

        $lastColumn = (3 + count($data));
        $sheet->getStyle('B4:H'. $lastColumn)->getAlignment()->setHorizontal(Alignment::HORIZONTAL_CENTER);
        $sheet->getStyle('I4:F'. $lastColumn)->getAlignment()->setHorizontal(Alignment::HORIZONTAL_LEFT);
        $sheet->getStyle('B4:F'. $lastColumn)->applyFromArray($this->generator->getBorderStyleBody());

        $sum = 0;
        foreach ($data as $attendance)
        {
            $when = new \Classes\DateTimeUtil($attendance[AManager::DATE]);

            $sheet->setCellValue($column.$row, $parameters[$parameterKey]);
            $sheet->setCellValue(++$column.$row, $attendance[AManager::WORK_FROM]);
            $sheet->setCellValue(++$column.$row, $attendance[AManager::WORK_TO]);
            $sheet->setCellValue(++$column.$row, $attendance[AManager::WORK_HOURS]);
            $sheet->setCellValue(++$column.$row, $when->getDateFormat());
            $row++;
            $column = 'B';
            $sum = $sum + $attendance[AManager::WORK_HOURS];
        }

        //$sumRange = 'E4:E4'.$lastColumn;
        $sheet->setCellValue('D'.$row , 'Celkem:');
        $sheet->setCellValue('E'.$row , $sum);

        $now = new \Classes\DateTimeUtil();
        $this->generator->save($now->getDateFormat() . '_OdpracovaneHodiny');
    }

    public function setParameters(array &$parameters, string $parameterKey) : void
    {
        switch($parameterKey)
        {
            case 'program':
                $parameters[$parameterKey] = AManager::PROGRAM_DICTIONARY_REVERSE[$parameters[$parameterKey]];
                break;
            case 'month':
                $parameters[$parameterKey] = DateTimeUtil::MONTHS_REVERSE[$parameters[$parameterKey]];
                break;
            case 'userName':
                if (preg_match('/ \((.*?)\)/', $parameters[$parameterKey], $match) == 1)
                {
                    $parameters['user'] = $this->loginManager->findAllByName($match[1])[LManager::PRIMARY];
                    unset($parameters[$parameterKey]);
                }
                break;
        }
    }
}