<?php
namespace Classes;


use Presenters\Container;
use Tools\Inflector;


class DefaultClass
{
    const ALERT_ERROR = 'warning';
    const ALERT_SUCCESS = 'success';

    const LATTE_ADMIN = 'admin/';
    const LATTE_USER = 'user/';


    const MENU_HOME = 1;
    const MENU_USER = 2;
    const MENU_REPORTS = 3;
    const MENU_ATTENDANCE = 4;
    const MENU_OLD_DATABASE = 5;

    public ?Container $container;
    protected $view, $reflection, $template;

    public function __construct(Container $container = null)
    {
        $this->container = $container;
    }

    public static function getInstance($container = null)
    {
        static $instance = null;
        if ($instance === null)
        {
            $instance = new static($container);
            $instance->reflection = new \ReflectionClass($instance);
        }
        return $instance;
    }

    /**
     * Navraci parametr z GET / POST
     * pokud je zadán klíč, vrátí pouze danou hodnotu
     * pokud není, vratí vše jak v POST tak i v GET
     *
     */
    public function getParameter(string $key = null)
    {
        $parameters = array_merge($_POST, $_GET);

        if ($key)
        {
            foreach ($parameters as $paramKey => $data)
            {
                if($key == $paramKey)
                {
                    return $data;
                }
            }
        }

        return $parameters;
    }

    public function setLatte() : \Latte\Engine
    {
        $latte = new \Latte\Engine;
        $latte->setTempDirectory('cache');

        $this->template = $this->getDefaultData();
        $this->isSelect2Used();

        return $latte;
    }

    public function init($args)
    {
        unset($args[0]);
        if(array_key_exists(2, $args))
        {
            unset($args[1]);
        }
        $view = strtolower(array_shift($args));

        if ($view == '')
        {
            $view = 'default';
        }

        $this->view = strtr($view, ['-' => '_']);
        $methodName = 'render' . Inflector::camelize($view);

        if ($this->reflection->hasMethod($methodName))
        {
            $result = $this->reflection->getMethod($methodName)->invokeArgs($this, $args);
            echo $result;

            exit;
        } else {
            if ($this->reflection->hasMethod('renderDefault'))
            {
                $result = $this->reflection->getMethod('renderDefault')->invokeArgs($this, $args);
                echo $result;

                exit;
            }
        }
    }

    private function getDefaultData() : array
    {
        $template['isLogged'] = StaticFunctions::checkLoggedIn();
        $template['messages'] = $this->getMessage();
        $template['userData'] = null;

        if ($template['isLogged'])
        {
            $template['userData'] = $this->container->getLoginManager()->findAllByName(StaticFunctions::getUserName());
            $template['userData']['permName'] = StaticFunctions::permissionDictionary($template['userData']['permission']);
        }

        return $template;
    }


    private function getMessage() : array
    {
        $return = [];

        if (array_key_exists('alert', $_SESSION))
        {
            foreach ($_SESSION['alert'] as $key => $messages)
            {
                foreach ($messages as $key => $message)
                {
                    $return[$key] = $message;
                }
            }
        }

        unset($_SESSION['alert']);
        return $return;
    }

    public function checkLoggedIn() : bool
    {
        return (array_key_exists('userName', $_SESSION));
    }


    public function renderLayout(\Latte\Engine $latte, string $file, array $template, string $specificLatte = '',
                                 bool $refreshMessage = false) : void
    {
        $url = 'www/' . $specificLatte;

        $url .= $file .'.latte';

        if($refreshMessage)
        {
            $template['messages'] = $this->getMessage();
        }

        $latte->render($url, $template);
    }

    public function addData(string $key, $data) : void
    {
        $this->template[$key] = $data;
    }

    public function getData(string $key = null)
    {
        if ($key)
        {
            return $this->template[$key];
        } else {
            return $this->template;
        }
    }

    public function isSelect2Used(bool $used = false) : void
    {
        $this->addData('select2', $used);
    }

    public function setActiveMenu(int $menuID) : void
    {
        $this->addData('activeMenu', $menuID);
    }

    public function setActiveModal(int $modalID) : void
    {
        $this->addData('modal', $modalID);
    }

    public function isLoggedIn(bool $checkIfAdmin = true) : void
    {
        if (!$this->template['isLogged'])
        {
            StaticFunctions::setMessage('Nejste přihlášen', self::ALERT_ERROR);
            StaticFunctions::setHeader('/');
            exit;
        }

        if ($checkIfAdmin && !StaticFunctions::isAdmin())
        {
            StaticFunctions::setMessage('Nemáte oprávnění do admin sekce', DefaultClass::ALERT_ERROR);
            $this->setActiveMenu(self::MENU_HOME);
            $this->renderLayout(StaticFunctions::getLatte(), 'index', $this->getData(), self::LATTE_USER);
            exit;
        }
    }

    public static function getLatte() : \Latte\Engine
    {
        return new \Latte\Engine;
    }

    public static function renderLatte(string $url, array $data) : string
    {
        return self::getLatte()->renderToString($url,$data);
    }
}
