<?php

namespace Classes\File;

use Classes\DateTimeUtil;
use Classes\File\Manager as FManager;
use Classes\Login\Manager as LManager;
use Classes\StaticFunctions;

class Facade
{
    private FManager $fileManager;
    private LManager $loginManager;

    public function __construct(FManager $fileManager, LManager $loginManager)
    {
        $this->fileManager = $fileManager;
        $this->loginManager = $loginManager;
    }

    public function uploadFile(int $userID, array $file)
    {
        $userData = $this->loginManager->getAllByPrimary($userID);

        $userName = $userData[LManager::USER_NAME];
        $userName = str_replace(' ', '', $userName);
        $target_dir = './www/files/' . $userName;
        if (!is_dir($target_dir))
        {
            mkdir($target_dir);
        }

        $fileName = $file['name'][0];
        $target_file = $target_dir . basename($fileName);

        $ext = pathinfo($target_file, PATHINFO_EXTENSION);
        $baseName = explode('.'.$ext ,$fileName)[0];
        $baseName = str_replace('-', '_', $baseName);

        $date = new DateTimeUtil();
        $newName = sprintf('%s-%s.%s', $baseName, $date->getTimestamp(), $ext);
        $new_file = $target_dir.'/'.$newName;
        $returnFile = sprintf('files/%s/%s',$userName, $newName);


        if (move_uploaded_file($file["tmp_name"][0], $new_file))
        {
            $this->fileManager->managerInsert([
                    FManager::PATH => $returnFile,
                    FManager::TYPE => strtolower($ext),
                    FManager::NAME => $newName,
                    FManager::SIZE => $file['size'][0],
                    FManager::FK_USER_ID => $userID,
                    FManager::FK_CONTRACT_ID => null
                ]
            );
        }
    }

    public function getFilesByUser(int $userID): string
    {
        $files = $this->fileManager->getAllByUserID($userID);

        return StaticFunctions::renderLatte('www/admin/tables/contracts.latte', ['files' => $files]);
    }

    public function deleteFile(int $fileID)
    {
        $data = $this->fileManager->getAllByPrimary($fileID);

        unlink('./www/' . $data['path']);

        $this->fileManager->deleteByPrimary($fileID);
    }
}