<?php
namespace Classes\Login;
use Classes\DateTimeUtil;
use Classes\DefaultClass;
use Classes\Login\Manager as LManager;
use Classes\Registration\Manager as RManager;
use Classes\StaticFunctions;
use Classes\StaticFunctions as Statics;

class Facade
{
    private RManager $registrationManager;
    private LManager $loginManager;

    public function __construct(LManager $loginManager, RManager $registrationManager)
    {
        $this->loginManager = $loginManager;
        $this->registrationManager = $registrationManager;
    }


    public function addUser(array $data) : bool
    {
        $exist = $this->loginManager->findByName($data[LManager::USER_NAME]);
        if (!$exist)
        {
            if ($data['password1'] === $data['password2'])
            {
                $encrypt = Statics::passwordEncrypt();
                $password = Statics::crypt($data['password1'], $encrypt);

                $login[LManager::USER_NAME] = $data[LManager::USER_NAME];
                $login[LManager::PASSWORD] = $password;
                $login[LManager::PASSWORD_ENCRYPT] = $encrypt;
                $login[LManager::CAN_ENTER] = 1;
                $login[LManager::PERMISSION] = LManager::PERMISSION_USER;
                $loginID = $this->loginManager->managerInsert($login, true);

                $date = new \DateTime($data[RManager::BIRTH_DATE]);
                $registration[RManager::FK_LOGIN_ID] = $loginID;
                $registration[RManager::FIRST_NAME] = $data[RManager::FIRST_NAME];
                $registration[RManager::SUR_NAME] = $data[RManager::SUR_NAME];
                $registration[RManager::STREET] = $data[RManager::STREET];
                $registration[RManager::HOUSE_NUMBER] = $data[RManager::HOUSE_NUMBER];
                $registration[RManager::POST] = $data[RManager::POST];
                $registration[RManager::ZIP_CODE] = $data[RManager::ZIP_CODE];
                $registration[RManager::BIRTH_DATE] = $date->format('Y-m-d');
                $registration[RManager::AGE] = $data[RManager::AGE];
                $registration[RManager::EMAIL] = $data[RManager::EMAIL];
                $registration[RManager::PHONE] = $data[RManager::PHONE];
                $this->registrationManager->managerInsert($registration);

                Statics::setMessage('Přidání proběhlo úspěšně.');
                return true;
            } else {
                Statics::setMessage('Hesla se neshodují.', DefaultClass::ALERT_ERROR);
                return false;
            }
        } else {
            Statics::setMessage('Toto uživatelské jméno již existuje. Pro zachování jedinečnosti jej prosím změňte',
                DefaultClass::ALERT_ERROR);
            return false;
        }
    }

    public function getAllVoluntiers(bool $generate = false)
    {
        $all = $this->loginManager->getAllVolunteers();

        foreach ($all as $key => $volunteer)
        {
            $date = new DateTimeUtil($volunteer[RManager::BIRTH_DATE]);
            $json = [
                LManager::USER_NAME => $volunteer[LManager::USER_NAME],
                RManager::PRIMARY => $volunteer[RManager::PRIMARY],
                RManager::FIRST_NAME => $volunteer[RManager::FIRST_NAME],
                RManager::SUR_NAME => $volunteer[RManager::SUR_NAME],
                RManager::EMAIL => $volunteer[RManager::EMAIL],
                RManager::STREET => $volunteer[RManager::STREET],
                RManager::HOUSE_NUMBER => $volunteer[RManager::HOUSE_NUMBER],
                RManager::POST => $volunteer[RManager::POST],
                RManager::ZIP_CODE => $volunteer[RManager::ZIP_CODE],
                RManager::BIRTH_DATE => $date->getDateFormat(),
                RManager::AGE => $volunteer[RManager::AGE],
                RManager::PHONE => $volunteer[RManager::PHONE]
            ];

            $all[$key]['permissionName'] = Statics::permissionDictionary($volunteer[LManager::PERMISSION]);
            $all[$key]['json'] = json_encode($json);
        }

        if ($generate)
        {
            return Statics::renderLatte('www/admin/tables/editUsers.latte', [
                    'allVolunteers' => $all
                ]
            );
        } else {
            return $all;
        }
    }

    public function changeEntry(int $userID) : string
    {
        $userData = $this->loginManager->getAllByPrimary($userID);

        $this->loginManager->managerUpdate([LManager::CAN_ENTER => intval(!$userData[LManager::CAN_ENTER])], $userID);

        return $this->getAllVoluntiers(true);
    }

    public function login($userName, $password)
    {
        $existingUser = $this->loginManager->findAllByName($userName);
        $err = [];
        if ($existingUser)
        {
            if ($existingUser[LManager::CAN_ENTER])
            {
                $passwordData = $this->loginManager->getPassword($existingUser[LManager::PRIMARY]);
                $encryptedPassword = StaticFunctions::crypt($password, $passwordData[LManager::PASSWORD_ENCRYPT]);

                if ($encryptedPassword === $passwordData[LManager::PASSWORD])
                {
                    $err = [
                        'msg' => 'Byl jste přihlášen/a',
                        'state' => true,
                        'userID' => $existingUser['id']
                    ];
                    if (in_array($existingUser[LManager::PERMISSION] , [LManager::PERMISSION_ADMIN]))
                    {
                        $err = [
                            'msg' => 'Aplikace je pouze pro dobrovolníky',
                            'state' => false
                        ];
                    }
                } else {
                    $err = [
                        'msg' => 'Špatné heslo',
                        'state' => false
                    ];
                }
            } else {
                StaticFunctions::setMessage('Nemáte oprávnění ke vstupu');
                $err = [
                    'msg' => 'Nemáte oprávnění ke vstupu',
                    'state' => false
                ];
            }

        } else {
            StaticFunctions::setMessage('Tento uživatel neexistuje');
            $err = [
                    'msg' => 'Tento uživatel neexistuje',
                    'state' => false
                ];
        }

        return $err;
    }
}