<?php
namespace Classes\Login;

class Manager extends \Classes\Dibi
{
    const TABLE = 'tLogin';
    const PRIMARY = 'id';

    const USER_NAME = 'userName';
    const PASSWORD = 'password';
    const PASSWORD_ENCRYPT = 'passwordEncrypt';
    const CAN_ENTER = 'canEnter';
    const PERMISSION = 'permission';
    const FK_ORGANIZATION_ID = 'FK_organizationID';

    const PERMISSION_GOD = 0;
    const PERMISSION_ADMIN = 1;
    const PERMISSION_USER = 2;

    public function findByName(string $userName) : bool
    {
        $query = $this->db->select(1);
        $query->from(self::TABLE);
        $query->where('%n = %s', self::USER_NAME, $userName);

        return (bool)$this->getFirst($query);
    }

    public function findAllByName(string $userName)
    {
        $query = $this->db->select('l.%n', self::PRIMARY);
        $query->select(self::PERMISSION);
        $query->select(self::CAN_ENTER);
        $query->select(self::USER_NAME);
        $query->select(self::FK_ORGANIZATION_ID);
        $query->select(\Classes\Registration\Manager::PHONE);
        $query->select(\Classes\Registration\Manager::EMAIL);
        $query->select(\Classes\Registration\Manager::AGE);
        $query->select(\Classes\Registration\Manager::BIRTH_DATE);
        $query->select(\Classes\Registration\Manager::ZIP_CODE);
        $query->select(\Classes\Registration\Manager::POST);
        $query->select(\Classes\Registration\Manager::HOUSE_NUMBER);
        $query->select(\Classes\Registration\Manager::STREET);
        $query->select(\Classes\Registration\Manager::FIRST_NAME);
        $query->select(\Classes\Registration\Manager::SUR_NAME);
        $query->from(self::TABLE)->as('l');
        $query->leftJoin(\Classes\Registration\Manager::TABLE)->as('r')->on('l.%n = r.%n',
            self::PRIMARY, \Classes\Registration\Manager::FK_LOGIN_ID);
        $query->where('%n = %s', self::USER_NAME, $userName);

        return $this->getFirst($query);
    }

    public function getPassword(int $userID)
    {
        $query = $this->db->select(self::PASSWORD);
        $query->select(self::PASSWORD_ENCRYPT);
        $query->from(self::TABLE);
        $query->where('%n = %i', self::PRIMARY, $userID);

        return $this->getFirst($query);
    }

    public function getAllVolunteers() : array
    {
        $query = $this->db->select('l.%n', self::PRIMARY);
        $query->select('l.%n', self::CAN_ENTER);
        $query->select('l.%n', self::PERMISSION);
        $query->select('l.%n', self::USER_NAME);
        $query->select('r.%n', self::ALL);
        $query->select('l.%n', self::PRIMARY)->as('loginID');
        $query->from(self::TABLE)->as('l');
        $query->leftJoin(\Classes\Registration\Manager::TABLE)->as('r')->on('l.%n = r.%n',
            self::PRIMARY, \Classes\Registration\Manager::FK_LOGIN_ID);
        $query->where('l.%n > %i', self::PERMISSION, self::PERMISSION_GOD);

        return $this->fetchAll($query);
    }
}