<?php
namespace Classes;


use DateInterval;
use DateTime;

class StaticFunctions
{
    public static function crypt($first, $second) : string
    {
        return hash_hmac("SHA512", $first, $second);
    }

    public static function passwordEncrypt(int $length = 50) : string
    {
        $alphabet = '1234567890abcdefghijklmnopqrstuvwxyz1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $pass = array();
        $alphaLength = strlen($alphabet) - 1;
        for ($i = 0; $i < $length; $i++)
        {
            $n = rand(0, $alphaLength);
            $pass[] = $alphabet[$n];
        }
        return implode($pass);
    }

    public static function setMessage(string $message, string $type = DefaultClass::ALERT_SUCCESS) : void
    {
        $array[$type] = $message;
        $_SESSION['alert'][] = $array;
    }

    public static function getUserID() : ?int
    {
        return (array_key_exists('userID', $_SESSION))? $_SESSION['userID'] : null;
    }

    public static function setUserID(int $id) : void
    {
        $_SESSION['userID'] = $id;
    }

    public static function getOrganization() : ?int
    {
        return (array_key_exists('organization', $_SESSION))? $_SESSION['organization'] : null;
    }

    public static function setOrganization(int $organization) : void
    {
        $_SESSION['organization'] = $organization;
    }

    public static function getPermission() : ?int
    {
        return (array_key_exists('permission', $_SESSION))? $_SESSION['permission'] : null;
    }

    public static function setPermission(int $permission) : void
    {
        $_SESSION['permission'] = $permission;
    }

    public static function getUserName() : string
    {
        return (array_key_exists('userName', $_SESSION))? $_SESSION['userName'] : '';
    }

    public static function setUserName(string $userName) : void
    {
        $_SESSION['userName'] = $userName;
    }

    public static function getWorkingHourReportType() : bool
    {
        return (array_key_exists('workingOurReport', $_SESSION))? $_SESSION['workingOurReport'] : true;
    }

    public static function setWorkingHourReportType() : void
    {
        $_SESSION['workingOurReport'] = !self::getWorkingHourReportType();
    }

    public static function getLatte() : \Latte\Engine
    {
        return new \Latte\Engine;
    }

    public static function isAdmin() : int
    {
        return (array_key_exists('admin', $_SESSION))? $_SESSION['admin'] : 0;
    }

    public static function setAdmin(int $isAdmin) : void
    {
        $_SESSION['admin'] = $isAdmin;
    }

    public static function renderLatte(string $url, array $data) : string
    {
        return StaticFunctions::getLatte()->renderToString($url, $data);
    }

    public static function setHeader(string $location) : void
    {
        header('Location: '. $location);
        exit;
    }

    public static function checkLoggedIn() : bool
    {
        return array_key_exists('userName', $_SESSION);
    }

    public static function permissionDictionary(int $permission) : string
    {
        $allPermissions = [
            Login\Manager::PERMISSION_GOD => 'Správce',
            Login\Manager::PERMISSION_ADMIN => 'Admin',
            Login\Manager::PERMISSION_USER => 'Uživatel',
        ];

        return $allPermissions[$permission];
    }

    public static function getDateFormat(DateTime $date) : string
    {
        return $date->format('Y-m-d');
    }
}