<?php

use Classes\StaticFunctions as Statics;
use Classes\Attendance\Manager as AManager;
use Classes\Registration\Manager as RManager;
use Classes\Login\Manager as LManager;
use PhpOffice\PhpSpreadsheet\Style\Alignment;

class Attendance extends \Classes\DefaultClass
{

    public function renderDefault()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn(false);
        $this->setActiveMenu(self::MENU_HOME);

        $this->addData('programs', \Classes\Attendance\Manager::PROGRAM_DICTIONARY);

        $this->renderLayout($latte, 'index', $this->getData(), self::LATTE_USER, true);
    }

    public function renderAddAttendance()
    {
        $createdDT = new \Classes\DateTimeUtil();
        $createdA = new \Classes\DateTimeUtil($this->getParameter('date'));

        $attendance[AManager::FK_USER_ID] = $this->getParameter('id');
        $attendance[AManager::PROGRAM] = $this->getParameter('program');
        $attendance[AManager::MONTH] = $createdA->format('m');
        $attendance[AManager::WORK_HOURS] = (float) $this->getParameter('workHours');
        $attendance[AManager::WORK_FROM] = $this->getParameter('workFrom');
        $attendance[AManager::WORK_TO] = $this->getParameter('workTo');
        $attendance[AManager::DESCRIPTION] = $this->getParameter('description');
        $attendance[AManager::DATE] = $createdA->getDateFormat();
        $attendance[AManager::YEAR] = $createdA->format('Y');
        $attendance[AManager::CREATE_DT] = $createdDT->getDateTimeFormat();
        $attendance[AManager::STATE] = 0;
        if ($this->getParameter('attendanceID'))
        {
            $this->container->getAttendanceManager()->managerUpdate($attendance, $this->getParameter('attendanceID'));
        } else {
            $this->container->getAttendanceManager()->managerInsert($attendance);
        }

        if (Statics::isAdmin())
        {
            Statics::setHeader('/admin/');
        } else {
            Statics::setHeader('/attendance/edit');
        }
    }

    public function renderEdit()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn(false);
        $this->setActiveMenu(self::MENU_USER);
        $this->setActiveModal(1);

        $page = 1;
        $attendanceData = $this->container->getAttendanceFacade()->getAttendanceForUser($page, 0);

        $this->addData('pages', $attendanceData['pages']);
        $this->addData('attendances', $attendanceData['data']);
        $this->addData('programs', \Classes\Attendance\Manager::PROGRAM_DICTIONARY);
        $this->addData('page', $page);
        $this->addData('isAdminPage', 0);

        $this->renderLayout($latte, 'attendance', $this->getData(), self::LATTE_USER, true);
    }

    public function renderReport()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn();
        $this->setActiveMenu(self::MENU_REPORTS);
        $this->isSelect2Used(true);

        $this->addData('programs', AManager::PROGRAM_DICTIONARY);
        $this->addData('months', \Classes\DateTimeUtil::getAllMonths());
        $this->addData('users', $this->container->getAttendanceManager()->getAllUsers());
        $this->addData('years', $this->container->getAttendanceManager()->getAllYears());

        $page = 1;
        $attendanceData = $this->container->getAttendanceFacade()->getAttendanceForReport($page);

        $this->addData('pages', $attendanceData['pages']);
        $this->addData('attendances', $attendanceData['data']);
        $this->addData('page', $page);
        $this->addData('isWorkingHour', 0);

        $this->renderLayout($latte, 'report', $this->getData(), self::LATTE_ADMIN);
    }

    public function renderWorkHours()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn();
        $this->setActiveMenu(self::MENU_REPORTS);
        $this->isSelect2Used(true);

        $this->addData('sum', Statics::getWorkingHourReportType());
        $this->addData('programs', AManager::PROGRAM_DICTIONARY);
        $this->addData('months', \Classes\DateTimeUtil::getAllMonths());
        $this->addData('users', $this->container->getAttendanceManager()->getAllUsers());
        $this->addData('years', $this->container->getAttendanceManager()->getAllYears());


        $this->renderLayout($latte, 'workHours', $this->getData(), self::LATTE_ADMIN);
    }

    public function renderGenerateReport()
    {
        $this->container->getAttendanceReports()->generateAttendanceReport($this->getParameter());
    }

    public function renderGenerateWorkingReport($type)
    {
        if ($type)
        {
            $this->container->getAttendanceReports()->generateWorkingReportStrict($this->getParameter());
        } else {
            $this->container->getAttendanceReports()->generateWorkingReportTable($this->getParameter());
        }
    }

}
