<?php
namespace Presenters;

use Classes\ExcelGenerator;
use dibi;
use ReflectionClass;
use ReflectionFunction;
use ReflectionMethod;
use RuntimeException;

class Container
{
    const MODULE = 'Module';

    private function getInstance()
    {
        $trace = debug_backtrace();
        $trace = $trace[$trace[1]['function'] == 'getManager' ? 2 : 1];
        $reflectionMethod = new ReflectionMethod($trace['class'], $trace['function']);

        $class = '\Classes';
        foreach (preg_split('/(?=[A-Z])/',$reflectionMethod->getName()) as $key => $name)
        {
            if (!$key) continue;
            $class .= trim(sprintf('\%s', $name));
        }

        if (!class_exists($class))
        {
            throw new RuntimeException($class . ' = does not exists');
        }

        static $instanceList = [];
        if (!array_key_exists($class, $instanceList))
        {
            $reflection = new ReflectionClass($class);
            $instanceList[$class] = $reflection->newInstanceArgs(func_get_args());
        }

        return $instanceList[$class];
    }

    public function getDibi() : Dibi\Connection
    {
        return dibi::getConnection();
    }

    public function getPaginator() : \Classes\Paginator
    {
        return $this->getInstance($this->getDibi());
    }

    public function getLoginManager() : \Classes\Login\Manager
    {
        return $this->getInstance($this->getDibi());
    }

    public function getLoginFacade() : \Classes\Login\Facade
    {
        return $this->getInstance($this->getLoginManager(), $this->getRegistrationManager());
    }

    public function getFileManager() : \Classes\File\Manager
    {
        return $this->getInstance($this->getDibi());
    }

    public function getFileFacade() : \Classes\File\Facade
    {
        return $this->getInstance($this->getFileManager(), $this->getLoginManager());
    }

    public function getRegistrationManager() : \Classes\Registration\Manager
    {
        return $this->getInstance($this->getDibi());
    }

    public function getOrganizationManager() : \Classes\Organization\Manager
    {
        return $this->getInstance($this->getDibi());
    }

    public function getAttendanceManager() : \Classes\Attendance\Manager
    {
        return $this->getInstance($this->getDibi());
    }

    public function getAttendanceFacade() : \Classes\Attendance\Facade
    {
        return $this->getInstance($this->getAttendanceManager(), $this->getPaginator());
    }

    public function getAttendanceReports() : \Classes\Attendance\Reports
    {
        return $this->getInstance($this->getAttendanceManager(), $this->getAttendanceFacade(), $this->getLoginManager(),
            new ExcelGenerator());
    }


    public function getOldManager() : \Classes\Old\Manager
    {
        return $this->getInstance($this->getDibi());
    }
}