<?php

use Classes\DateTimeUtil;
use Classes\Login\Manager as LManager;
use Classes\Registration\Manager as RManager;
use Classes\Attendance\Manager as AManager;
use Classes\StaticFunctions;

class DatabaseConverter extends \Classes\DefaultClass
{

    public function renderDefault()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn();
        $this->setActiveMenu(self::MENU_OLD_DATABASE);


        $this->renderLayout($latte, 'oldDatabase', $this->getData(), self::LATTE_ADMIN, true);
    }

    public function renderOldLoginToNew()
    {
        $oldLoginData = $this->container->getOldManager()->oldLoginData();
        $newLoginData = $this->container->getLoginManager()->getAll();

        $existDataCheck = [];
        foreach ($newLoginData as $newData)
        {
            $existDataCheck[$newData[LManager::USER_NAME]] = $newData;
        }

        $dataToSave = $dataToSaveTemp = [];
        foreach ($oldLoginData as $dataOld)
        {
            if (in_array($dataOld['jmeno_u'], $existDataCheck))
            {
                continue;
            }

            $dataToSave[LManager::USER_NAME] = $dataOld['jmeno_u'];
            $dataToSave[LManager::PASSWORD] = 'a';
            $dataToSave[LManager::PASSWORD_ENCRYPT] = 'a';
            $dataToSave[LManager::CAN_ENTER] = 1;
            $dataToSave[LManager::PERMISSION] = $dataOld['opravneni'] == 0 ? LManager::PERMISSION_USER : LManager::PERMISSION_ADMIN;
            $dataToSave['temporary'] = substr($dataOld['heslo'], 0, 8);
            $dataToSave[LManager::FK_ORGANIZATION_ID] = 1;

            $this->container->getLoginManager()->managerInsert($dataToSave);
            $dataToSaveTemp[] = $dataToSave;
        }

        //dumpe($dataToSaveTemp);
        StaticFunctions::setHeader('/databaseConverter/');
    }

    public function renderOldRegistrationToNew()
    {
        $oldRegistrationData = $this->container->getOldManager()->oldRegistrationData();
        $newLoginData = $this->container->getLoginManager()->getAll(LManager::USER_NAME);

        $dataToSave = $dataToSaveTemp = [];
        foreach ($oldRegistrationData as $oldData)
        {
            $dataToSave[RManager::FK_LOGIN_ID] = $newLoginData[$oldData['jmeno_u']][LManager::PRIMARY];
            $dataToSave[RManager::FIRST_NAME] = $oldData['jmeno'];
            $dataToSave[RManager::SUR_NAME] = $oldData['prijmeni'];
            $dataToSave[RManager::STREET] = $oldData['bydliste'];
            $dataToSave[RManager::HOUSE_NUMBER] = $oldData['cislo_domu'];
            $dataToSave[RManager::POST] = $oldData['posta'];
            $dataToSave[RManager::ZIP_CODE] = $oldData['psc'];
            $dataToSave[RManager::AGE] = $oldData['vek'];
            $dataToSave[RManager::EMAIL] = $oldData['email'];
            $dataToSave[RManager::PHONE] = $oldData['telefon'];

            $this->container->getRegistrationManager()->managerInsert($dataToSave);
            $dataToSaveTemp[] = $dataToSave;
        }

        //dump($dataToSaveTemp);
        //die;
        StaticFunctions::setHeader('/databaseConverter/');
    }

    public function renderOldAttendanceToNew()
    {
        $oldAttendanceData = $this->container->getOldManager()->oldAttendanceData();
        $newLoginData = $this->container->getLoginManager()->getAll(LManager::USER_NAME);

        $dataToSave = $dataToSaveTemp = [];
        foreach ($oldAttendanceData as $oldData)
        {

            if (! array_key_exists($oldData['jmeno_u'], $newLoginData))
            {
                $newLoginData[$oldData['jmeno_u']][LManager::PRIMARY] = ($this->addUnknownUser($oldData['jmeno_u']));
            }

            $createDT = new DateTimeUtil($oldData['datum_vyt']);
            $dataToSave[AManager::FK_USER_ID] = $newLoginData[$oldData['jmeno_u']][LManager::PRIMARY];
            $dataToSave[AManager::PROGRAM] = $this->getProgram($oldData['program'], $oldData['jmeno_u']);
            $dataToSave[AManager::MONTH] = DateTimeUtil::MONTHS_REVERSE[$this->utf8Upper($oldData['mesic'])];
            $dataToSave[AManager::WORK_HOURS] = $oldData['odprac_h'];
            $dataToSave[AManager::WORK_FROM] = $oldData['od'];
            $dataToSave[AManager::WORK_TO] = $oldData['do'];
            $dataToSave[AManager::DESCRIPTION] = $oldData['prace'];
            $dataToSave[AManager::DATE] = new DateTime($oldData['cas']);
            $dataToSave[AManager::YEAR] = $oldData['rok'];
            $dataToSave[AManager::CREATE_DT] = $createDT->getDateFormat();
            $dataToSave[AManager::STATE] = ($oldData['stav'] == 'images/false.png') ? 0 : 1;

            $this->container->getAttendanceManager()->managerInsert($dataToSave);
            $dataToSaveTemp[] = $dataToSave;
        }

        //dump($dataToSaveTemp);
    }

    private function utf8Upper(string $str) : string
    {
        $fc = mb_strtoupper(mb_substr($str, 0, 1));
        return $fc.mb_substr($str, 1);
    }

    private function getProgram(string $program, string $name) : ?int
    {
        if (array_key_exists($program, AManager::PROGRAM_DICTIONARY_REVERSE_OLD))
        {
            return AManager::PROGRAM_DICTIONARY_REVERSE_OLD[$program];
        }

        switch($program)
        {
            case 'K2-04-2016':
                return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['K2'];
            break;
            case 'N80 - 1 - 2017':
            case 'N80 - 2 - 2017':
                return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['N80'];
            break;
            case 'pět P':
            case 'Program Pět P':
            case 'Pět P':
                return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['PětP'];
            break;
            case '3':
            case '1':
                return intval($program);
            break;
            default:
                switch ($name)
                {
                    case 'Vladislav':
                    case 'Lucie':
                        return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['K2'];
                    break;
                    case 'Klára':
                    case 'Denisa':
                    case 'Kaja':
                        return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['PětP'];
                    break;
                    case 'Hynek':
                        return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['N80'];
                    break;
                    case 'Míša':
                        return AManager::PROGRAM_DICTIONARY_REVERSE_OLD['B612'];
                    break;
                    default:
                        return null;
                    break;
                }
            break;
        }

    }

    private function addUnknownUser(string $name) : int
    {
        $loginInsert[LManager::USER_NAME] = $name;
        $loginInsert[LManager::PASSWORD] = 'a';
        $loginInsert[LManager::PASSWORD_ENCRYPT] = 'a';
        $loginInsert[LManager::CAN_ENTER] = 0;
        $loginInsert[LManager::PERMISSION] = LManager::PERMISSION_USER;
        $loginInsert[LManager::FK_ORGANIZATION_ID] = 1;

        $loginID = $this->container->getLoginManager()->managerInsert($loginInsert, true);

        $registerInsert[RManager::FK_LOGIN_ID] = $loginID;
        $registerInsert[RManager::FIRST_NAME] = $name;
        $registerInsert[RManager::SUR_NAME] = 'unknown';
        $registerInsert[RManager::STREET] = 'unknown';
        $registerInsert[RManager::HOUSE_NUMBER] = 'unknown';
        $registerInsert[RManager::POST] = 'unknown';
        $registerInsert[RManager::ZIP_CODE] = 0;
        $registerInsert[RManager::BIRTH_DATE] = null;
        $registerInsert[RManager::AGE] = null;
        $registerInsert[RManager::EMAIL] = 'unknown@unknown.cz';
        $registerInsert[RManager::PHONE] = 0;

        $this->container->getRegistrationManager()->managerInsert($registerInsert);

        return $loginID;
    }
}