<?php

use Classes\DefaultClass;
use Classes\Login\Manager as LManager;
use Classes\Registration\Manager as RManager;
use Classes\StaticFunctions;

class User extends DefaultClass
{
    public function renderAddUser()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn();
        $this->setActiveMenu(Admin::MENU_USER);

        if ($_POST)
        {
            if (!$this->container->getLoginFacade()->addUser($this->getParameter()))
            {
                $this->addData('userDataAdd', $this->getParameter());
            }
        }

        $this->renderLayout($latte, 'addUser', $this->getData(), self::LATTE_ADMIN, true);
    }

    public function renderLogin()
    {
        $userName = $this->getParameter('username');
        $password = $this->getParameter('password');
        $existingUser = $this->container->getLoginManager()->findAllByName($userName);
        if ($existingUser)
        {
            if ($existingUser[LManager::CAN_ENTER])
            {
                $passwordData = $this->container->getLoginManager()->getPassword($existingUser[LManager::PRIMARY]);
                $encryptedPassword = StaticFunctions::crypt($password, $passwordData[LManager::PASSWORD_ENCRYPT]);

                if ($encryptedPassword === $passwordData[LManager::PASSWORD])
                {
                    StaticFunctions::setMessage('Byl jste přihlášen');
                    StaticFunctions::setUserName($userName);
                    StaticFunctions::setPermission($existingUser[LManager::PERMISSION]);
                    StaticFunctions::setUserID($existingUser[LManager::PRIMARY]);
                    StaticFunctions::setOrganization($existingUser[LManager::FK_ORGANIZATION_ID]);
                    StaticFunctions::setAdmin(0);

                    if ($existingUser[LManager::PERMISSION] < 2)
                    {
                        StaticFunctions::setAdmin(1);
                        StaticFunctions::setHeader('/admin/');
                    }
                    StaticFunctions::setHeader('/attendance/');
                } else {
                    StaticFunctions::setMessage('Špatné heslo');
                    StaticFunctions::setHeader('/');
                }
            } else {
                StaticFunctions::setMessage('Nemáte oprávnění ke vstupu');
                StaticFunctions::setHeader('/');
            }
           
        } else {
            StaticFunctions::setMessage('Tento uživatel neexistuje');
            StaticFunctions::setHeader('/');
        }
    }

    public function renderLogout()
    {
        session_unset();
        session_destroy();
        unset($_SESSION);

        session_start();
        StaticFunctions::setMessage('Byl jste odhlášen');

        StaticFunctions::setHeader('/');
    }

    public function renderEditMe()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn(false);
        $this->setActiveMenu(0);

        if (array_key_exists('editPassword', $this->getParameter()))
        {
            if (array_key_exists('password1', $this->getParameter()))
            {
                $this->changePassword();
            }
        }

        if (array_key_exists('editData', $this->getParameter()))
        {
            $data = $this->getParameter();
            unset($data['editData']);
            $this->container->getRegistrationManager()->updateRegistrationData($data);
            StaticFunctions::setMessage('Data byla upravena');
        }

        $this->addData('userDatUpdate', $this->container->getRegistrationManager()->getDatToEdit(StaticFunctions::getUserID()));

        $this->renderLayout($latte, 'editMe', $this->getData(), self::LATTE_ADMIN, true);
    }

    public function renderEditUser()
    {
        $latte = $this->setLatte();
        $this->isLoggedIn();
        $this->setActiveMenu(self::MENU_USER);
        $this->setActiveModal(2);

        $this->addData('allVolunteers', $this->container->getLoginFacade()->getAllVoluntiers());

        $this->renderLayout($latte, 'editUser', $this->getData(), self::LATTE_ADMIN);
    }

    private function changePassword()
    {
        $ps1 = $this->getParameter('password1');
        $ps2 = $this->getParameter('password2');

        if ($ps1 === $ps2)
        {
            $encode = StaticFunctions::passwordEncrypt();
            $data[LManager::PASSWORD] = StaticFunctions::crypt($ps1, $encode);
            $data[LManager::PASSWORD_ENCRYPT] = $encode;

            StaticFunctions::setMessage('Heslo bylo úspěšně změněno');
        } else {
            StaticFunctions::setMessage('Hesla se neshodují', DefaultClass::ALERT_ERROR);
        }
    }

    public function renderDeleteUser($id)
    {
        $this->container->getLoginManager()->deleteByPrimary($id);
        $this->container->getRegistrationManager()->deleteUserData($id);
        $this->container->getAttendanceManager()->deleteUserData($id);

        StaticFunctions::setMessage('Uživatel a jeho data byly úspěšně smazánay');
        StaticFunctions::setHeader('/user/editUser');
    }

    public function renderEditUserData()
    {
        $volunteer = $this->getParameter();
        $data = [
            RManager::FIRST_NAME => $volunteer[RManager::FIRST_NAME],
            RManager::SUR_NAME => $volunteer[RManager::SUR_NAME],
            RManager::EMAIL => $volunteer[RManager::EMAIL],
            RManager::STREET => $volunteer[RManager::STREET],
            RManager::HOUSE_NUMBER => $volunteer[RManager::HOUSE_NUMBER],
            RManager::POST => $volunteer[RManager::POST],
            RManager::ZIP_CODE => $volunteer[RManager::ZIP_CODE],
            RManager::BIRTH_DATE => $volunteer[RManager::BIRTH_DATE],
            RManager::AGE => $volunteer[RManager::AGE],
            RManager::PHONE => $volunteer[RManager::PHONE]

        ];

        $this->container->getRegistrationManager()->managerUpdate($data, $volunteer['id']);

        StaticFunctions::setMessage(sprintf('Data uživate %s %s byla změněna',
            $data[RManager::FIRST_NAME], $data[RManager::SUR_NAME]));
        StaticFunctions::setHeader('/user/editUser');
    }
}